<?php

namespace FireDrum;

if ( !defined( 'ABSPATH' ) ) {
	die();
}

/**
 * Bloom Provider
 */
class Email_Marketing_ET_Core_API_Email_Provider extends \ET_Core_API_Email_Provider {
	/**
	 * @inheritDoc
	 */
	public $custom_fields_scope = 'account';

	/**
	 * @inheritDoc
	 */
	public $uses_oauth = false;

	private $plugin;
	private $API_URL_REQUIRED = '';

	public function _set_http_timeout($timeout) {
		return 60;
	}

	public function __construct($owner = '', $account_name = '', $api_key = '') {
		$this->plugin = Email_Marketing_Plugin::instantiate();

		$this->name = Email_Marketing_Plugin::get_menu_branding();
		$this->slug = Email_Marketing_Plugin::get_branding();

		parent::__construct( Email_Marketing_Plugin::get_textdomain() . '/' . Email_Marketing_Plugin::get_version(), $account_name, $api_key );

		$this->http->expects_json = true;

		$this->API_URL_REQUIRED = esc_html__( 'API URL is required.', Email_Marketing_Plugin::get_textdomain() );
	}

	protected function _fetch_custom_fields( $list_id = '', $list = array() ) {
		$this->response_data_key = 'customFields';
		$this->prepare_request( $this->plugin->get_api()->get_custom_field_list_url( $this->data['api_url'] ), 'POST', false, array(
			'authAPIKey' => $this->data['api_key']
		) );

		add_filter( 'http_request_timeout', array (
			$this,
			'_set_http_timeout'
		) );
		$result = parent::_fetch_custom_fields( $list_id, $list );
		remove_filter( 'http_request_timeout', array (
			$this,
			'_set_http_timeout'
		) );

		/* 
		 * Modify the custom field options returned from the API to be
		 * id => name pairs instead of simple strings.
		 */
		foreach ( $results as &$field ) {
			if ( isset( $field['options'] ) ) {
				$options = array();

				foreach ( $field['options'] as $option ) {
					$options[ $option ] = $option;
				}

				$field['options'] = $options;
			}
		}

		return $result;
	}

	protected function _process_custom_fields( $args ) {
		if ( ! isset( $args['customFields[]'] ) ) {
			return $args;
		}

		$rawFields = $args['customFields[]'];

		unset( $args['customFields[]'] );

		$customFields = array();
		foreach ( $rawFields as $id => $value ) {
			$customFields[] = json_encode( array(
				'id' => $id,
				'value' => $value
			) );
		}

		$args['customFields[]'] = $customFields;

		return $args;
	}

	/**
	 * @inheritDoc
	 */
	public function get_account_fields() {
		return array (
			'api_key' => array (
				'label' => esc_html__( 'API Key', Email_Marketing_Plugin::get_textdomain() ),
				'apply_password_mask' => false
			),
			'api_url' => array(
				'label' => esc_html__( 'API URL', Email_Marketing_Plugin::get_textdomain() ),
				'apply_password_mask' => false
			),
		);
	}

	/**
	 * @inheritDoc
	 */
	public function get_data_keymap($keymap = array(), $custom_fields_key = '') {
		$keymap = array (
			'list' => array (
				'list_id' => 'id',
				'name' => 'name',
				'subscribers_count' => 'totalMemberCount'
			),
			'subscriber' => array (
				'email' => 'email',
				'last_name' => 'lastName',
				'name' => 'firstName',
				'list_id' => 'categoryId[]',
				'custom_fields' => 'customField[]'
			),
			'error' => array (
				'error_message' => 'responseCode.message'
			),
			'custom_field' => array(
				'field_id' => 'id',
				'name' => 'name',
				'type' => 'dataType',
				'hidden' => '!publicDisplay',
				'options' => 'options',
			),
			/*
			 * Unfortunately we do not return custom field options as objects.
			 * We need to convert the API response to an object for this to work correctly.
			 * This is left here for forwards compatibility
			 */
			'custom_field_option' => array(
				'id' => 'id',
				'name' => 'name',
			),
			'custom_field_type'   => array(
				'radio' => 'radio',
				'select' => 'select',
				'input' => 'text',
				/* If they ever add date field support...
				'date' => 'date'
				*/
			),
		);

		return parent::get_data_keymap( $keymap, $custom_fields_key );
	}

	/**
	 * @inheritDoc
	 */
	public function fetch_subscriber_lists() {
		if ( empty( $this->data['api_key'] ) ) {
			return $this->API_KEY_REQUIRED;
		}

		if ( empty( $this->data['api_url'] ) ) {
			return $this->API_URL_REQUIRED;
		}

		/**
		 * The maximum number of subscriber lists to request from the API.
		 *
		 * @since 2.0.0
		 * @param int $max_lists
		 */
		$max_lists = ( int ) apply_filters( 'et_core_api_email_' . Email_Marketing_Plugin::get_branding() . '_max_lists', 100 );

		$this->prepare_request( $this->plugin->get_api()->get_category_list_url( $this->data['api_url'] ), 'POST', false, array(
			'authAPIKey' => rawurlencode( $this->data['api_key'] ),
			'limit' => rawurlencode( $max_lists ),
			'includeMemberCount' => 'true'
		) );
		$this->request->HEADERS['Content-Type'] = 'application/x-www-form-urlencoded';
		$this->response_data_key = 'categories';

		add_filter( 'http_request_timeout', array (
			$this,
			'_set_http_timeout'
		) );
		$result = parent::fetch_subscriber_lists();
		remove_filter( 'http_request_timeout', array (
			$this,
			'_set_http_timeout'
		) );

		return $result;
	}

	/**
	 * @inheritDoc
	 */
	public function subscribe($args, $url = '') {
		if ( empty( $this->data['api_key'] ) ) {
			return $this->API_KEY_REQUIRED;
		}

		if ( empty( $this->data['api_url'] ) ) {
			return $this->API_URL_REQUIRED;
		}

		$url = $this->plugin->get_api()->get_add_contact_url( $this->data['api_url'] );
		$args = array_merge( array (
			'authAPIKey' => rawurlencode( $this->data['api_key'] ),
			'updateIfExists' => 'true',
			'skipCaptcha' => 'true'
		), apply_filters( Email_Marketing_Plugin::get_branding() . '_bloom_subscribe', $this->transform_data_to_provider_format( $args, 'subscriber' ) ) );

		$args = $this->_process_custom_fields( $args );

		$this->prepare_request( $url, 'POST', false, $args );
		$this->request->HEADERS['Content-Type'] = 'application/x-www-form-urlencoded';

		add_filter( 'http_request_timeout', array (
			$this,
			'_set_http_timeout'
		) );
		$result = parent::subscribe( $args, $url );
		remove_filter( 'http_request_timeout', array (
			$this,
			'_set_http_timeout'
		) );

		return $result;
	}
}
